from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['Codecov']


class Codecov(ActionStep):
    """Upload coverage reports to Codecov.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    base_sha
        The base SHA to select. Only used in the ``pr-base-picking`` run command.
    binary
        The file location of a pre-downloaded version of the CLI. If specified,
        integrity checking will be bypassed.
    codecov_yml_path
        The location of the codecov.yml file. This is currently only used for
        automated test selection.
    commit_parent
        SHA (with 40 chars) of what should be the parent of this commit.
    directory
        Folder to search for coverage files. Defaults to the current working
        directory.
    disable_file_fixes
        Disable file fixes to ignore common lines from coverage.
    disable_search
        Disable search for coverage files.
    disable_safe_directory
        Disable setting safe directory. Set to true to disable.
    disable_telem
        Disable sending telemetry data to Codecov. Set to true to disable.
    dry_run
        Do not upload files to Codecov.
    env_vars
        Environment variables to tag the upload with (e.g. ``PYTHON`` or
        ``OS,PYTHON``).
    exclude
        Comma-separated list of folders to exclude from search.
    fail_ci_if_error
        On error, exit with non-zero code.
    files
        Comma-separated list of explicit files to upload. These will be added to
        coverage files found for upload. Use ``disable_search`` to upload only
        the specified files.
    flags
        Comma-separated list of flags to upload to group coverage metrics.
    force
        Only used for the ``empty-upload`` run command.
    git_service
        Override the git_service (e.g. ``github_enterprise``).
    gcov_args
        Extra arguments to pass to gcov.
    gcov_executable
        gcov executable to run. Defaults to ``gcov``.
    gcov_ignore
        Paths to ignore during gcov gathering.
    gcov_include
        Paths to include during gcov gathering.
    handle_no_reports_found
        If no coverage reports are found, do not raise an exception.
    job_code
        Codecov job code.
    codecov_name
        Custom defined name of the upload. Visible in the Codecov UI.
    network_filter
        Filter files listed in the network section of the Codecov report. Only
        files whose path begin with the specified filter are included.
    network_prefix
        Prefix on files listed in the network section of the Codecov report.
    os
        Override the assumed OS. Options available at cli.codecov.io.
    override_branch
        Specify the branch to be displayed with this commit on Codecov.
    override_build
        Specify the build number manually.
    override_build_url
        The URL of the build where this is running.
    override_commit
        Commit SHA (with 40 chars).
    override_pr
        Specify the pull request number manually. Used to override pre-existing
        CI environment variables.
    plugins
        Comma-separated list of plugins to run. Specify ``noop`` to turn off all
        plugins.
    recurse_submodules
        Whether to enumerate files inside of submodules for path-fixing purposes.
    report_code
        The code of the report if using local upload.
    report_type
        The type of file to upload. Possible values are ``test_results`` and
        ``coverage``.
    root_dir
        Root folder from which to consider paths on the network section.
    run_command
        Choose which CLI command to run. Options are ``upload-coverage``,
        ``empty-upload``, ``pr-base-picking``, and ``send-notifications``.
    skip_validation
        Skip integrity checking of the CLI. This is not recommended.
    slug
        Required when using the org token. Set to the owner/repo slug used
        instead of the private repo token.
    swift_project
        Specify the swift project name. Useful for optimization.
    token
        Repository Codecov token. Used to authorize report uploads.
    url
        Set to the Codecov instance URL. Used by Dedicated Enterprise Cloud
        customers.
    use_legacy_upload_endpoint
        Use the legacy upload endpoint.
    use_oidc
        Use OIDC instead of token. This will ignore any token supplied.
    use_pypi
        Use the PyPI version of the CLI instead of from cli.codecov.io.
    verbose
        Enable verbose logging.
    codecov_version
        Which version of the Codecov CLI to use (defaults to ``latest``).
    working_directory
        Directory in which to execute codecov.sh.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated codecov step.

    See Also
    --------
    GitHub repository: https://github.com/codecov/codecov-action
    """

    recommended_permissions = None

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v5',
        base_sha: Ostrlike = None,
        binary: Ostrlike = None,
        codecov_yml_path: Ostrlike = None,
        commit_parent: Ostrlike = None,
        directory: Ostrlike = None,
        disable_file_fixes: Oboollike = None,
        disable_search: Oboollike = None,
        disable_safe_directory: Oboollike = None,
        disable_telem: Oboollike = None,
        dry_run: Oboollike = None,
        env_vars: Ostrlike = None,
        exclude: Ostrlike = None,
        fail_ci_if_error: Oboollike = None,
        files: Ostrlike = None,
        flags: Ostrlike = None,
        force: Oboollike = None,
        git_service: Ostrlike = None,
        gcov_args: Ostrlike = None,
        gcov_executable: Ostrlike = None,
        gcov_ignore: Ostrlike = None,
        gcov_include: Ostrlike = None,
        handle_no_reports_found: Oboollike = None,
        job_code: Ostrlike = None,
        codecov_name: Ostrlike = None,
        network_filter: Ostrlike = None,
        network_prefix: Ostrlike = None,
        os: Ostrlike = None,  #
        override_branch: Ostrlike = None,
        override_build: Ointlike = None,
        override_build_url: Ostrlike = None,
        override_commit: Ostrlike = None,
        override_pr: Ointlike = None,
        plugins: Ostrlike = None,
        recurse_submodules: Oboollike = None,
        report_code: Ostrlike = None,
        report_type: Ostrlike = None,
        root_dir: Ostrlike = None,
        run_command: Ostrlike = None,
        skip_validation: Oboollike = None,
        slug: Ostrlike = None,
        swift_project: Ostrlike = None,
        token: Ostrlike = None,
        url: Ostrlike = None,
        use_legacy_upload_endpoint: Oboollike = None,
        use_oidc: Oboollike = None,
        use_pypi: Oboollike = None,
        verbose: Oboollike = None,
        codecov_version: Ostrlike = None,
        working_directory: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> Codecov:
        options: dict[str, object] = {
            'base_sha': base_sha,
            'binary': binary,
            'codecov_yml_path': codecov_yml_path,
            'commit_parent': commit_parent,
            'directory': directory,
            'disable_file_fixes': disable_file_fixes,
            'disable_search': disable_search,
            'disable_safe_directory': disable_safe_directory,
            'disable_telem': disable_telem,
            'dry_run': dry_run,
            'env_vars': env_vars,
            'exclude': exclude,
            'fail_ci_if_error': fail_ci_if_error,
            'files': files,
            'flags': flags,
            'force': force,
            'git_service': git_service,
            'gcov_args': gcov_args,
            'gcov_executable': gcov_executable,
            'gcov_ignore': gcov_ignore,
            'gcov_include': gcov_include,
            'handle_no_reports_found': handle_no_reports_found,
            'job_code': job_code,
            'name': codecov_name,
            'network_filter': network_filter,
            'network_prefix': network_prefix,
            'os': validate_choice(
                'os',
                os,
                ['alpine', 'alpine-arm64', 'linux', 'linux-arm64', 'macos', 'windows'],
            ),
            'override_branch': override_branch,
            'override_build': override_build,
            'override_build_url': override_build_url,
            'override_commit': override_commit,
            'override_pr': override_pr,
            'plugins': plugins,
            'recurse_submodules': recurse_submodules,
            'report_code': report_code,
            'report_type': validate_choice(
                'report_type', report_type, ['test_results', 'coverage']
            ),
            'root_dir': root_dir,
            'run_command': validate_choice(
                'run_command',
                run_command,
                [
                    'upload-coverage',
                    'empty-upload',
                    'pr-base-picking',
                    'send-notifications',
                ],
            ),
            'skip_validation': skip_validation,
            'slug': slug,
            'swift_project': swift_project,
            'token': token,
            'url': url,
            'use_legacy_upload_endpoint': use_legacy_upload_endpoint,
            'use_oidc': use_oidc,
            'use_pypi': use_pypi,
            'verbose': verbose,
            'version': codecov_version,
            'working-directory': working_directory,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Upload coverage'

        return super().__new__(
            cls,
            name,
            'codecov/codecov-action',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom import Permissions
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
    BoolLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['SetupNode', 'SetupPnpm']


class SetupNode(ActionStep):
    """Set up a Node.js environment.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    node_version
        Version spec of the version to use (e.g. ``12.x``).
    node_version_file
        File containing the version spec to use (e.g. ``.nvmrc``).
    architecture
        Target architecture for Node to use.
    check_latest
        Check for the latest available version that satisfies the version spec.
    registry_url
        Optional registry to set up for auth.
    scope
        Optional scope for authenticating against scoped registries.
    token
        Used to pull Node distributions from node-versions.
    cache
        Package manager for caching in the default directory. Supported values:
        ``npm``, ``yarn``, ``pnpm``.
    package_manager_cache
        Set to false to disable automatic caching.
    cache_dependency_path
        Path to a dependency file (package-lock.json, yarn.lock, etc.).
    mirror
        Alternative mirror to download Node.js binaries from.
    mirror_token
        Token used as Authorization header when fetching from the mirror.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-node step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-node
    """

    recommended_permissions = Permissions(contents='read')

    @classmethod
    def cache_hit(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['cache-hit']

    @classmethod
    def node_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['node-version']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v6',
        node_version: Ostrlike = None,
        node_version_file: Ostrlike = None,
        check_latest: Oboollike = None,
        architecture: Ostrlike = None,
        token: Ostrlike = None,
        cache: Ostrlike = None,
        package_manager_cache: Oboollike = None,
        cache_dependency_path: Ostrlike = None,
        registry_url: Ostrlike = None,
        scope: Ostrlike = None,
        mirror: Ostrlike = None,
        mirror_token: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupNode:
        options: dict[str, object] = {
            'node-version': node_version,
            'node-version-file': node_version_file,
            'check-latest': check_latest,
            'architecture': architecture,
            'token': token,
            'cache': validate_choice('cache', cache, ['npm', 'yarn', 'pnpm']),
            'package-manager-cache': package_manager_cache,
            'cache-dependency-path': cache_dependency_path,
            'registry-url': registry_url,
            'scope': scope,
            'mirror': mirror,
            'mirror-token': mirror_token,
        }

        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup Node'

        return super().__new__(
            cls,
            name,
            'actions/setup-node',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )


class SetupPnpm(ActionStep):
    """Set up pnpm.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    pnpm_version
        The pnpm version to use.
    dest
        The destination for pnpm.
    run_install
        Whether to run pnpm install.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-pnpm step.

    See Also
    --------
    GitHub repository: https://github.com/pnpm/action-setup
    """

    recommended_permissions = None

    @classmethod
    def dest(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.dest

    @classmethod
    def bin_dest(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.bin_dest

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v4',
        pnpm_version: Ostrlike = None,
        dest: Ostrlike = None,
        run_install: StringLike | BoolLike | None = None,
        cache: Oboollike = None,
        cache_dependency_path: Ostrlike | list[StringLike] = None,
        package_json_file: Ostrlike = None,
        standalone: Oboollike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupPnpm:
        options: dict[str, object] = {
            'version': pnpm_version,
            'dest': dest,
            'run_install': run_install,
            'cache': cache,
            'cache_dependency_path': cache_dependency_path,
            'package_json_file': package_json_file,
            'standalone': standalone,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup pnpm'

        return super().__new__(
            cls,
            name,
            'pnpm/action-setup',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom import Permissions
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['SetupJava']


class SetupJava(ActionStep):
    """Set up a specific version of the Java JDK and add tools to PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    java_version
        The Java version to set up. Takes a whole or semver Java version.
    java_version_file
        Path to the ``.java-version`` file.
    distribution
        Java distribution (required).
    java_package
        The package type (``jdk``, ``jre``, ``jdk+fx``, ``jre+fx``).
    architecture
        The architecture of the package.
    jdk_file
        Path to where the compressed JDK is located.
    check_latest
        Check for the latest available version that satisfies the version spec.
    server_id
        ID of the distributionManagement repository in the pom.xml file.
    server_username
        Environment variable name for the username for authentication to the
        Apache Maven repository.
    server_password
        Environment variable name for password or token for authentication to
        the Apache Maven repository.
    settings_path
        Path to where the settings.xml file will be written.
    overwrite_settings
        Overwrite the settings.xml file if it exists.
    gpg_private_key
        GPG private key to import.
    gpg_passphrase
        Environment variable name for the GPG private key passphrase.
    cache
        Name of the build platform to cache dependencies (``maven``, ``gradle``,
        ``sbt``).
    cache_dependency_path
        Path to a dependency file such as pom.xml, build.gradle, build.sbt, etc.
        Supports wildcards or a list of file names for caching multiple
        dependencies.
    mvn_toolchain_id
        Name of Maven Toolchain ID if the default name is not wanted.
    mvn_toolchain_vendor
        Name of Maven Toolchain Vendor if the default name is not wanted.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-java step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-java
    """

    recommended_permissions = Permissions(contents='read')

    @classmethod
    def distribution(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.distribution

    @classmethod
    def java_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['java-version']

    @classmethod
    def java_home(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['java-home']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v5',
        distribution: Ostrlike = None,
        java_version_file: Ostrlike = None,
        java_version: Ostrlike = None,
        java_package: Ostrlike = None,
        check_latest: Oboollike = None,
        architecture: Ostrlike = None,
        jdk_file: Ostrlike = None,
        cache: Ostrlike = None,
        cache_dependency_path: Ostrlike = None,
        overwrite_settings: Oboollike = None,
        server_id: Ostrlike = None,
        server_username: Ostrlike = None,
        server_password: Ostrlike = None,
        settings_path: Ostrlike = None,
        gpg_private_key: Ostrlike = None,
        gpg_passphrase: Ostrlike = None,
        mvn_toolchain_id: Ostrlike = None,
        mvn_toolchain_vendor: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupJava:
        options: dict[str, object] = {
            'java-version': java_version,
            'java-version-file': java_version_file,
            'distribution': distribution,
            'java-package': validate_choice(
                'java_package', java_package, ['jdk', 'jre', 'jdk+fx', 'jre+fx']
            ),
            'check-latest': check_latest,
            'architecture': validate_choice(
                'architecture',
                architecture,
                ['x86', 'x64', 'armv7', 'aarch64', 'ppc64le'],
            ),
            'jdkFile': jdk_file,
            'cache': validate_choice('cache', cache, ['maven', 'gradle', 'sbt']),
            'cache-dependency-path': cache_dependency_path,
            'overwrite-settings': overwrite_settings,
            'server-id': server_id,
            'server-username': server_username,
            'server-password': server_password,
            'settings-path': settings_path,
            'gpg-private-key': gpg_private_key,
            'gpg-passphrase': gpg_passphrase,
            'mvn-toolchain-id': mvn_toolchain_id,
            'mvn-toolchain-vendor': mvn_toolchain_vendor,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup Java'

        return super().__new__(
            cls,
            name,
            'actions/setup-java',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom import Permissions

from typing import TYPE_CHECKING

from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['Maturin', 'PypiPublish']


class Maturin(ActionStep):
    """Install and run a custom maturin command.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    token
        Used to pull maturin distributions using GitHub API.
    command
        Maturin command to run. Defaults to ``build``.
    args
        Arguments to pass to the maturin subcommand.
    maturin_version
        The version of maturin to use.
    manylinux
        Control the manylinux platform tag on Linux.
    container
        Manylinux Docker container image name. Set to ``off`` to disable
        manylinux Docker build and build on the host instead.
    docker_options
        Additional Docker run options, for passing environment variables, etc.
    host_home_mount
        Host folder where the runner's home folder is mounted to. Used for
        building using Docker-in-Docker runners.
    target
        The ``--target`` option for Cargo.
    rust_toolchain
        Rust toolchain name.
    rustup_components
        Rustup components.
    working_directory
        Working directory to run maturin in.
    sccache
        Enable sccache for faster builds.
    before_script_linux
        Script to run before maturin command on Linux.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated maturin step.

    See Also
    --------
    GitHub repository: https://github.com/PyO3/maturin-action
    """

    recommended_permissions = None

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v1',
        token: Ostrlike = None,
        command: Ostrlike = None,
        args: Ostrlike = None,
        maturin_version: Ostrlike = None,
        manylinux: Ostrlike = None,
        container: Ostrlike = None,
        docker_options: Ostrlike = None,
        host_home_mount: Ostrlike = None,
        target: Ostrlike = None,
        rust_toolchain: Ostrlike = None,
        rustup_components: list[StringLike] | None = None,
        working_directory: Ostrlike = None,
        sccache: Oboollike = None,
        before_script_linux: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> Maturin:
        options: dict[str, object] = {
            'token': token,
            'command': command,
            'maturin-version': maturin_version,
            'manylinux': manylinux,
            'target': target,
            'container': container,
            'docker-options': docker_options,
            'host-home-mount': host_home_mount,
            'rust-toolchain': rust_toolchain,
            'rustup-components': ','.join(str(s) for s in rustup_components)
            if rustup_components is not None
            else None,
            'working-directory': working_directory,
            'sccache': sccache,
            'before-script-linux': before_script_linux,
        }

        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Maturin Action'

        return super().__new__(
            cls,
            name,
            'PyO3/maturin-action',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )


class PypiPublish(ActionStep):
    """Upload Python distribution packages to PyPI.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    user
        PyPI user. Defaults to ``__token__``.
    password
        Password for your PyPI user or an access token.
    repository_url
        The repository URL to use.
    packages_dir
        The target directory for distribution.
    verify_metadata
        Check metadata before uploading.
    skip_existing
        Do not fail if a Python package distribution exists in the target package
        index.
    verbose
        Show verbose output.
    print_hash
        Show hash values of files to be uploaded.
    attestations
        Enable support for PEP 740 attestations. Only works with PyPI and TestPyPI
        via Trusted Publishing.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated PyPI publish step.

    See Also
    --------
    GitHub repository: https://github.com/pypa/gh-action-pypi-publish
    """

    recommended_permissions = Permissions(id_token='write')

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'release/v1',
        user: Ostrlike = None,
        password: Ostrlike = None,
        repository_url: Ostrlike = None,
        packages_dir: Ostrlike = None,
        verify_metadata: Oboollike = None,
        skip_existing: Oboollike = None,
        verbose: Oboollike = None,
        print_hash: Oboollike = None,
        attestations: Oboollike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> PypiPublish:
        options: dict[str, object] = {
            'user': user,
            'password': password,
            'repository-url': repository_url,
            'packages-dir': packages_dir,
            'verify-metadata': verify_metadata,
            'skip-existing': skip_existing,
            'verbose': verbose,
            'print-hash': print_hash,
            'attestations': attestations,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Publish to PyPI'

        return super().__new__(
            cls,
            name,
            'pypa/gh-action-pypi-publish',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom.actions.utils import validate_choice, check_string

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['SetupMPI']


class SetupMPI(ActionStep):
    """Set up a specific MPI implementation.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    mpi
        MPI implementation name.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-mpi step.

    See Also
    --------
    GitHub repository: https://github.com/mpi4py/setup-mpi
    """

    recommended_permissions = None

    @classmethod
    def mpi_home(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['mpi-home']

    @classmethod
    def mpi_bin(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['mpi-bin']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v1',
        mpi: Ostrlike = None,
        package: Ostrlike = None,
        mpifc: Ostrlike = None,
        mpif90: Ostrlike = None,
        mpif77: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupMPI:
        options: dict[str, object] = {
            'mpi': validate_choice(
                'mpi', mpi, ['mpich', 'openmpi', 'intelmpi', 'msmpi']
            ),
        }

        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            mpi_str = check_string(options.get('mpi'))
            if mpi_str:
                name = f'Setup {mpi_str}'
            else:
                name = 'Setup MPI'

        return super().__new__(
            cls,
            name,
            'mpi-action/setup-mpi',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom import Permissions

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['SetupGo']


class SetupGo(ActionStep):
    """Set up a Go environment and add it to the PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    go_version
        The Go version to download (if necessary) and use. Supports semver spec
        and ranges.
    go_version_file
        Path to the go.mod, go.work, .go-version, or .tool-versions file.
    check_latest
        Check for the latest available version that satisfies the version spec.
    token
        Used to pull Go distributions from go-versions.
    cache
        Enable caching for Go.
    cache_dependency_path
        Path to a dependency file (e.g., ``go.sum``).
    architecture
        Target architecture for Go to use.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-go step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-go
    """

    recommended_permissions = Permissions(contents='read')

    @classmethod
    def go_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['go-version']

    @classmethod
    def cache_hit(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['cache-hit']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v6',
        go_version: Ostrlike = None,
        go_version_file: Ostrlike = None,
        check_latest: Oboollike = None,
        architecture: Ostrlike = None,
        token: Ostrlike = None,
        cache: Oboollike = None,
        cache_dependency_path: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupGo:
        options: dict[str, object] = {
            'go-version': go_version,
            'go-version-file': go_version_file,
            'check-latest': check_latest,
            'architecture': architecture,
            'token': token,
            'cache': cache,
            'cache-dependency-path': cache_dependency_path,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup Go'

        return super().__new__(
            cls,
            name,
            'actions/setup-go',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom import Permissions
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringOrBoolLike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['SetupPython', 'SetupUV']


class SetupPython(ActionStep):
    """Set up a specific version of Python and add it to the PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    python_version
        Version range or exact version of Python or PyPy to use.
    python_version_file
        File containing the Python version to use.
    cache
        Package manager for caching in the default directory. Supported values:
        ``pip``, ``pipenv``, ``poetry``.
    architecture
        The target architecture of the Python or PyPy interpreter.
    check_latest
        Check for the latest available version that satisfies the version spec.
    token
        Token used to authenticate when fetching Python distributions.
    cache_dependency_path
        Path to dependency files. Supports wildcards or a list of file names.
    update_environment
        Set this option to update environment variables.
    allow_prereleases
        Allow prerelease versions to satisfy version range.
    freethreaded
        Use the freethreaded version of Python.
    pip_versions
        The version of pip to install with Python.
    pip_install
        Packages to install with pip after setting up Python.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-python step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-python
    """

    recommended_permissions = Permissions(contents='read')

    @classmethod
    def python_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['python-version']

    @classmethod
    def cache_hit(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['cache-hit']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v6',
        python_version: StringLike | None = None,
        python_version_file: Ostrlike = None,
        check_latest: Oboollike = None,
        architecture: Ostrlike = None,
        update_environment: Oboollike = None,
        token: Ostrlike = None,
        cache: Ostrlike = None,
        cache_dependency_path: Ostrlike = None,
        allow_prereleases: Oboollike = None,
        freethreaded: Oboollike = None,
        pip_version: Ostrlike = None,
        pip_install: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupPython:
        options: dict[str, object] = {
            'python-version': python_version,
            'python-version-file': python_version_file,
            'check-latest': check_latest,
            'architecture': architecture,
            'token': token,
            'cache': cache,
            'cache-dependency-path': cache_dependency_path,
            'update-environment': update_environment,
            'allow-prereleases': allow_prereleases,
            'freethreaded': freethreaded,
            'pip-version': pip_version,
            'pip-install': pip_install,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup Python'

        return super().__new__(
            cls,
            name,
            'actions/setup-python',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )


class SetupUV(ActionStep):
    """Set up a specific version of uv.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    uv_version
        The version of uv to install.
    uv_version_file
        Path to a file containing the version of uv to install.
    python_version
        The version of Python to set UV_PYTHON to.
    activate_environment
        Use uv venv to activate a venv ready to be used by later steps.
    working_directory
        The directory to execute all commands in and look for files.
    checksum
        The checksum of the uv version to install.
    github_token
        Used to increase the rate limit when retrieving versions and downloading uv.
    enable_cache
        Enable uploading of the uv cache. Accepts ``true``, ``false``, or ``auto``.
    cache_dependency_glob
        Glob pattern list to control the cache.
    restore_cache
        Whether to restore the cache if found.
    save_cache
        Whether to save the cache after the run.
    cache_suffix
        Suffix for the cache key.
    cache_local_path
        Local path to store the cache.
    prune_cache
        Prune cache before saving.
    cache_python
        Upload managed Python installations to the GitHub Actions cache.
    ignore_nothing_to_cache
        Ignore when nothing is found to cache.
    ignore_empty_workdir
        Ignore when the working directory is empty.
    tool_dir
        Custom path to set UV_TOOL_DIR to.
    tool_bin_dir
        Custom path to set UV_TOOL_BIN_DIR to.
    manifest_file
        URL to the manifest file containing available versions and download URLs.
    add_problem_matchers
        Add problem matchers.
    resolution_strategy
        Resolution strategy when resolving version ranges (``highest`` or
        ``lowest``).
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-uv step.

    See Also
    --------
    GitHub repository: https://github.com/astral-sh/setup-uv
    """

    recommended_permissions = None

    @classmethod
    def uv_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['uv-version']

    @classmethod
    def cache_hit(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['cache-hit']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v7',
        uv_version: Ostrlike = None,
        uv_version_file: Ostrlike = None,
        resolution_strategy: Ostrlike = None,
        python_version: Ostrlike = None,
        activate_environment: Oboollike = None,
        working_directory: Ostrlike = None,
        checksum: Ostrlike = None,
        github_token: Ostrlike = None,
        enable_cache: StringOrBoolLike | None = None,
        cache_dependency_glob: list[StringLike] | None = None,
        restore_cache: Oboollike = None,
        save_cache: Oboollike = None,
        cache_suffix: Ostrlike = None,
        cache_local_path: Ostrlike = None,
        prune_cache: Oboollike = None,
        cache_python: Oboollike = None,
        ignore_nothing_to_cache: Oboollike = None,
        ignore_empty_workdir: Oboollike = None,
        tool_dir: Ostrlike = None,
        tool_bin_dir: Ostrlike = None,
        manifest_file: Ostrlike = None,
        add_problem_matchers: Oboollike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupUV:
        options: dict[str, object] = {
            'version': uv_version,
            'version-file': uv_version_file,
            'resolution-strategy': validate_choice(
                'resolution_strategy', resolution_strategy, ['highest', 'lowest']
            ),
            'python-version': python_version,
            'activate-environment': activate_environment,
            'working-directory': working_directory,
            'checksum': checksum,
            'github-token': github_token,
            'enable-cache': enable_cache,
            'cache-dependency-glob': '\n'.join(str(s) for s in cache_dependency_glob)
            if cache_dependency_glob is not None
            else None,
            'restore-cache': restore_cache,
            'save-cache': save_cache,
            'cache-suffix': cache_suffix,
            'cache-local-path': cache_local_path,
            'prune-cache': prune_cache,
            'cache-python': cache_python,
            'ignore-nothing-to-cache': ignore_nothing_to_cache,
            'ignore-empty-workdir': ignore_empty_workdir,
            'tool-dir': tool_dir,
            'tool-bin-dir': tool_bin_dir,
            'manifest-file': manifest_file,
            'add-problem-matchers': add_problem_matchers,
        }
        if enable_cache is not None:
            if isinstance(enable_cache, str):
                if enable_cache.lower() != 'auto':
                    msg = "'enable_cache' must be 'auto', true or false"
                    raise ValueError(msg)
                options['enable-cache'] = 'auto'
            elif isinstance(enable_cache, bool):
                options['enable-cache'] = enable_cache
            else:
                raise TypeError('enable_cache must be a bool or a string')

        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup uv'

        return super().__new__(
            cls,
            name,
            'astral-sh/setup-uv',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom import Permissions
from yamloom.actions.utils import check_string

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Obool,
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringOrBoolLike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = [
    'Release',
    'ReleasePlease',
]


class Release(ActionStep):
    """Create a GitHub release.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    body
        Note-worthy description of changes in the release.
    body_path
        Path to load note-worthy description of changes in the release from.
    draft
        Creates a draft release.
    prerelease
        Identify the release as a prerelease.
    preserve_order
        Preserve the order of the artifacts when uploading.
    files
        Newline-delimited list of path globs for asset files to upload.
    working_directory
        Base directory to resolve ``files`` globs against (defaults to job
        working-directory).
    overwrite_files
        Overwrite existing files with the same name.
    release_name
        Gives the release a custom name. Defaults to tag name.
    tag_name
        Gives a tag name. Defaults to ``github.ref_name``.
    fail_on_unmatched_files
        Fail if any of the ``files`` globs match nothing.
    repository
        Repository to make releases against, in ``<owner>/<repo>`` format.
    target_commitish
        Commitish value that determines where the Git tag is created from. Can be
        any branch or commit SHA.
    token
        Authorized secret GitHub Personal Access Token.
    discussion_category_name
        If specified, a discussion of the specified category is created and linked
        to the release. The value must be a category that already exists in the
        repository.
    generate_release_notes
        Whether to automatically generate the name and body for this release. If
        name is specified, the specified name will be used; otherwise, a name will
        be automatically generated. If body is specified, the body will be
        pre-pended to the automatically generated notes.
    append_body
        Append to existing body instead of overwriting it.
    make_latest
        Specifies whether this release should be set as the latest release for the
        repository. Can be ``true``, ``false``, or ``legacy``. Uses GitHub API
        defaults if not provided.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated release step.

    See Also
    --------
    GitHub repository: https://github.com/softprops/action-gh-release
    """

    recommended_permissions = Permissions(contents='write')

    @classmethod
    def url(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.url

    @classmethod
    def release_id(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.id

    @classmethod
    def upload_url(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.upload_url

    @classmethod
    def assets(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.assets

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v2',
        body: Ostr = None,
        body_path: Ostr = None,
        draft: Obool = None,
        prerelease: Obool = None,
        preserve_order: Obool = None,
        files: Ostr = None,
        working_directory: Ostr = None,
        overwrite_files: Obool = None,
        release_name: Ostr = None,
        tag_name: Ostr = None,
        fail_on_unmatched_files: Obool = None,
        repository: Ostr = None,
        target_commitish: Ostr = None,
        token: Ostr = None,
        discussion_category_name: Ostr = None,
        generate_release_notes: Obool = None,
        append_body: Obool = None,
        make_latest: StringOrBoolLike | None = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> Release:
        options: dict[str, object] = {
            'body': body,
            'body_path': body_path,
            'draft': draft,
            'prerelease': prerelease,
            'preserve_order': preserve_order,
            'files': files,
            'working_directory': working_directory,
            'overwrite_files': overwrite_files,
            'name': release_name,
            'tag_name': tag_name,
            'fail_on_unmatched_files': fail_on_unmatched_files,
            'repository': repository,
            'target_commitish': target_commitish,
            'token': token,
            'discussion_category_name': discussion_category_name,
            'generate_release_notes': generate_release_notes,
            'append_body': append_body,
            'make_latest': make_latest,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            repository_str = check_string(options.get('repository'))
            if repository_str:
                name = f'Release {repository_str}'
            else:
                name = 'Create Release'

        recommended_permissions = (
            cls.recommended_permissions
            if discussion_category_name is None
            else Permissions(contents='write', discussions='write')
        )

        return super().__new__(
            cls,
            name,
            'softprops/action-gh-release',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=recommended_permissions if token is None else None,
        )


class ReleasePlease(ActionStep):
    """Automated releases based on conventional commits.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    token
        GitHub token for creating and grooming release PRs.
    release_type
        Defines the release strategy to use for the repository.
    path
        Create a release from a path other than the repository's root.
    target_branch
        Branch to open pull release PR against (detected by default).
    config_file
        Path to the release-please config in the repository.
    manifest_file
        Path to the release-please versions manifest.
    repo_url
        GitHub repository name in the form of ``<owner>/<repo>``.
    github_api_url
        Override the GitHub API URL.
    github_graphql_url
        Override the GitHub GraphQL URL.
    fork
        If true, send the PR from a fork. This requires the token to be a user
        that can create forks (not the default ``GITHUB_TOKEN``).
    include_component_in_tag
        If true, add prefix to tags and branches, allowing multiple libraries
        to be released from the same repository.
    proxy_server
        Configure a proxy server in the form of ``<host>:<port>``.
    skip_github_release
        If true, do not attempt to create releases.
    skip_github_pull_request
        If true, do not attempt to create release pull requests.
    skip_labeling
        If true, do not attempt to label the PR.
    changelog_host
        The proto://host where commits live.
    versioning_strategy
        The versioning strategy to use.
    release_as
        The version to release as.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated release-please step.

    Notes
    -----
    Root component outputs are only present when the release-please action is
    configured for the root package. This means the ``path`` input is either
    omitted or set to ``"."``. If a non-root component path is used, access the
    path-prefixed outputs via the ``*_for`` methods instead.

    See Also
    --------
    GitHub repository: https://github.com/googleapis/release-please-action

    Notes
    -----
    You may have to adjust repository settings to allow GitHub actions to create pull requests: ``Settings > Actions > General``
    """

    recommended_permissions = Permissions(
        contents='write', issues='write', pull_requests='write'
    )

    @classmethod
    def releases_created(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.releases_created

    @classmethod
    def paths_released(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.paths_released

    @classmethod
    def prs_created(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.prs_created

    @classmethod
    def pr(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.pr

    @classmethod
    def prs(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.prs

    @classmethod
    def release_created(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.release_created

    @classmethod
    def upload_url(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.upload_url

    @classmethod
    def html_url(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.html_url

    @classmethod
    def tag_name(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.tag_name

    @classmethod
    def version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.version

    @classmethod
    def major(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.major

    @classmethod
    def minor(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.minor

    @classmethod
    def patch(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.patch

    @classmethod
    def sha(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.sha

    @classmethod
    def body(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.body

    @classmethod
    def release_created_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--release_created`` output for a component path."""
        return context.steps[id].outputs[f'{path}--release_created']

    @classmethod
    def upload_url_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--upload_url`` output for a component path."""
        return context.steps[id].outputs[f'{path}--upload_url']

    @classmethod
    def html_url_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--html_url`` output for a component path."""
        return context.steps[id].outputs[f'{path}--html_url']

    @classmethod
    def tag_name_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--tag_name`` output for a component path."""
        return context.steps[id].outputs[f'{path}--tag_name']

    @classmethod
    def version_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--version`` output for a component path."""
        return context.steps[id].outputs[f'{path}--version']

    @classmethod
    def major_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--major`` output for a component path."""
        return context.steps[id].outputs[f'{path}--major']

    @classmethod
    def minor_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--minor`` output for a component path."""
        return context.steps[id].outputs[f'{path}--minor']

    @classmethod
    def patch_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--patch`` output for a component path."""
        return context.steps[id].outputs[f'{path}--patch']

    @classmethod
    def sha_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--sha`` output for a component path."""
        return context.steps[id].outputs[f'{path}--sha']

    @classmethod
    def body_for(cls, id: str, path: str) -> StringExpression:
        """Return ``<path>--body`` output for a component path."""
        return context.steps[id].outputs[f'{path}--body']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v4',
        token: Ostrlike = None,
        release_type: Ostrlike = None,
        path: Ostrlike = None,
        target_branch: Ostrlike = None,
        config_file: Ostrlike = None,
        manifest_file: Ostrlike = None,
        repo_url: Ostrlike = None,
        github_api_url: Ostrlike = None,
        github_graphql_url: Ostrlike = None,
        fork: Oboollike = None,
        include_component_in_tag: Oboollike = None,
        proxy_server: Ostrlike = None,
        skip_github_release: Oboollike = None,
        skip_github_pull_request: Oboollike = None,
        skip_labeling: Oboollike = None,
        changelog_host: Ostrlike = None,
        versioning_strategy: Ostrlike = None,
        release_as: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> ReleasePlease:
        options: dict[str, object] = {
            'token': token,
            'release-type': release_type,
            'path': path,
            'target-branch': target_branch,
            'config-file': config_file,
            'manifest-file': manifest_file,
            'repo-url': repo_url,
            'github-api-url': github_api_url,
            'github-graphql-url': github_graphql_url,
            'fork': fork,
            'include-component-in-tag': include_component_in_tag,
            'proxy-server': proxy_server,
            'skip-github-release': skip_github_release,
            'skip-github-pull-request': skip_github_pull_request,
            'skip-labeling': skip_labeling,
            'changelog-host': changelog_host,
            'versioning-strategy': versioning_strategy,
            'release-as': release_as,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Run release-please'

        return super().__new__(
            cls,
            name,
            'googleapis/release-please-action',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions
            if token is None
            else None,
        )



from yamloom.actions.toolchains.rust import SetupRust
from dataclasses import dataclass
from yamloom.actions.github.artifacts import DownloadArtifact, UploadArtifact
from yamloom.actions.github.release import ReleasePlease
from yamloom.actions.packaging.python import Maturin
from yamloom.actions.toolchains.python import SetupPython, SetupUV
from yamloom.actions.github.scm import Checkout
from yamloom.expressions import context
from yamloom import (
    Workflow,
    Events,
    PushEvent,
    PullRequestEvent,
    WorkflowDispatchEvent,
    Permissions,
    Job,
    Matrix,
    Strategy,
    script,
    Environment,
)


@dataclass
class Target:
    runner: str
    target: str
    skip_python_versions: list[str] | None = None


DEFAULT_PYTHON_VERSIONS = [
    '3.9',
    '3.10',
    '3.11',
    '3.12',
    '3.13',
    '3.13t',
    '3.14',
    '3.14t',
    'pypy3.11',
]


def resolve_python_versions(skip: list[str] | None) -> list[str]:
    if not skip:
        return DEFAULT_PYTHON_VERSIONS
    skipped = set(skip)
    return [version for version in DEFAULT_PYTHON_VERSIONS if version not in skipped]


def create_build_job(
    job_name: str, name: str, targets: list[Target], *, needs: list[str]
) -> Job:
    def platform_entry(target: Target) -> dict[str, object]:
        entry = {
            'runner': target.runner,
            'target': target.target,
            'python_versions': resolve_python_versions(target.skip_python_versions),
        }
        python_arch = (
            ('arm64' if target.target == 'aarch64' else target.target)
            if name == 'windows'
            else None
        )
        if python_arch is not None:
            entry['python_arch'] = python_arch
        return entry

    return Job(
        steps=[
            Checkout(),
            script(
                f'printf "%s\n" {context.matrix.platform.python_versions.as_array().join(" ")} >> version.txt',
            ),
            SetupPython(
                python_version_file='version.txt',
                architecture=context.matrix.platform.python_arch.as_str()
                if name == 'windows'
                else None,
            ),
            Maturin(
                name='Build wheels',
                target=context.matrix.platform.target.as_str(),
                args=f'--release --out dist --interpreter {context.matrix.platform.python_versions.as_array().join(" ")}',
                sccache=~context.github.ref.startswith('refs/tags/'),
                manylinux='musllinux_1_2'
                if name == 'musllinux'
                else ('auto' if name == 'linux' else None),
            ),
            UploadArtifact(
                path='dist',
                artifact_name=f'wheels-{name}-{context.matrix.platform.target}',
            ),
        ],
        runs_on=context.matrix.platform.runner.as_str(),
        strategy=Strategy(
            fast_fail=False,
            matrix=Matrix(
                platform=[platform_entry(target) for target in targets],
            ),
        ),
        needs=needs,
        condition=context.github.ref.startswith('refs/tags/')
        | (context.github.event_name == 'workflow_dispatch'),
    )


release_workflow = Workflow(
    name='Build and Release',
    on=Events(
        push=PushEvent(branches=['main'], tags=['*']),
        pull_request=PullRequestEvent(),
        workflow_dispatch=WorkflowDispatchEvent(),
    ),
    permissions=Permissions(contents='read'),
    jobs={
        'build-test-check': Job(
            steps=[
                Checkout(),
                SetupRust(components=['clippy']),
                SetupUV(python_version='3.9'),
                script('cargo clippy'),
                script('cargo test'),
                script(
                    'uv venv',
                    '. .venv/bin/activate',
                    'echo PATH=$PATH >> $GITHUB_ENV',
                    'uvx maturin develop --uv',
                ),
                script('uv pip install pytest'),
                script('uvx ruff check'),
                script('uvx ty check'),
                script('uv run pytest'),
            ],
            runs_on='ubuntu-latest',
        ),
        'linux': create_build_job(
            'Build Linux Wheels',
            'linux',
            [
                Target(
                    'ubuntu-22.04',
                    target,
                )
                for target in [
                    'x86_64',
                    'x86',
                    'aarch64',
                    'armv7',
                    's390x',
                    'ppc64le',
                ]
            ],
            needs=['build-test-check'],
        ),
        'musllinux': create_build_job(
            'Build (musl) Linux Wheels',
            'musllinux',
            [
                Target(
                    'ubuntu-22.04',
                    target,
                )
                for target in [
                    'x86_64',
                    'x86',
                    'aarch64',
                    'armv7',
                ]
            ],
            needs=['build-test-check'],
        ),
        'windows': create_build_job(
            'Build Windows Wheels',
            'windows',
            [
                Target(
                    'windows-latest',
                    'x64',
                ),
                Target('windows-latest', 'x86', ['pypy3.11']),
                Target(
                    'windows-11-arm',
                    'aarch64',
                    ['3.9', '3.10', '3.11', '3.13t', '3.14t', 'pypy3.11'],
                ),
            ],
            needs=['build-test-check'],
        ),
        'macos': create_build_job(
            'Build macOS Wheels',
            'macos',
            [
                Target(
                    'macos-15-intel',
                    'x86_64',
                ),
                Target(
                    'macos-latest',
                    'aarch64',
                ),
            ],
            needs=['build-test-check'],
        ),
        'sdist': Job(
            steps=[
                Checkout(),
                Maturin(name='Build sdist', command='sdist', args='--out dist'),
                UploadArtifact(path='dist', artifact_name='wheels-sdist'),
            ],
            name='Build Source Distribution',
            runs_on='ubuntu-22.04',
            needs=['build-test-check'],
            condition=context.github.ref.startswith('refs/tags/')
            | (context.github.event_name == 'workflow_dispatch'),
        ),
        'release': Job(
            steps=[
                DownloadArtifact(),
                SetupUV(),
                script(
                    'uv publish --trusted-publishing always wheels-*/*',
                ),
            ],
            name='Release',
            runs_on='ubuntu-22.04',
            condition=context.github.ref.startswith('refs/tags/')
            | (context.github.event_name == 'workflow_dispatch'),
            needs=['linux', 'musllinux', 'windows', 'macos', 'sdist'],
            permissions=Permissions(id_token='write', contents='write'),
            environment=Environment('pypi'),
        ),
    },
)

version_workflow = Workflow(
    name='Release Please',
    on=Events(
        push=PushEvent(
            branches=['main'],
        ),
    ),
    permissions=Permissions(contents='write', issues='write', pull_requests='write'),
    jobs={
        'release-please': Job(
            steps=[
                ReleasePlease(
                    token=context.secrets.RELEASE_PLEASE,
                    config_file='release-please-config.json',
                    manifest_file='.release-please-manifest.json',
                )
            ],
            runs_on='ubuntu-latest',
        )
    },
)


if __name__ == '__main__':
    release_workflow.dump('.github/workflows/release.yml')
    version_workflow.dump('.github/workflows/release-please.yml')



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['SetupRuby']


class SetupRuby(ActionStep):
    """Set up Ruby, JRuby, or TruffleRuby and add it to the PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    ruby_version
        Engine and version to use. If unset, reads from `.ruby-version`,
        `.tool-versions`, or `mise.toml`. Use ``default`` to let the action pick.
    rubygems
        RubyGems version to use: ``default``, ``latest``, or a version number.
    bundler
        Bundler version to install: ``Gemfile.lock`` (default), ``default``,
        ``latest``, ``none``, or a version number.
    bundler_cache
        Run ``bundle install`` and cache the result automatically.
    working_directory
        Working directory for resolving Ruby version files and `Gemfile.lock`.
    cache_version
        String added to the bundler cache key to force cache invalidation.
    self_hosted
        Treat the runner as self-hosted to avoid prebuilt binaries.
    windows_toolchain
        Override Windows toolchain setup. Allowed values: ``default`` or ``none``.
    token
        GitHub token used to authenticate downloads when required.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-ruby step.

    See Also
    --------
    GitHub repository: https://github.com/ruby/setup-ruby
    """

    recommended_permissions = None

    @classmethod
    def ruby_prefix(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['ruby-prefix']

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v1',
        ruby_version: Ostrlike = None,
        rubygems: Ostrlike = None,
        bundler: Ostrlike = None,
        bundler_cache: Oboollike = None,
        working_directory: Ostrlike = None,
        cache_version: Ostrlike = None,
        self_hosted: Oboollike = None,
        windows_toolchain: Ostrlike = None,
        token: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupRuby:
        options: dict[str, object] = {
            'ruby-version': ruby_version,
            'rubygems': rubygems,
            'bundler': bundler,
            'bundler-cache': bundler_cache,
            'working-directory': working_directory,
            'cache-version': cache_version,
            'self-hosted': self_hosted,
            'windows-toolchain': validate_choice(
                'windows-toolchain', windows_toolchain, ['default', 'none']
            ),
            'token': token,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup Ruby'

        return super().__new__(
            cls,
            name,
            'ruby/setup-ruby',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import ActionStep
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['InstallRustTool', 'SetupRust']


class SetupRust(ActionStep):
    """Set up Rust toolchains with optional caching.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    toolchain
        Comma-separated list of Rust toolchain specifications. The last version
        becomes the default.
    target
        Target triple to install for this toolchain.
    components
        Comma-separated list of components to install.
    cache
        Automatically configure Rust cache.
    cache_workspaces
        Paths to multiple Cargo workspaces and their target directories,
        separated by newlines.
    cache_directories
        Additional non-workspace directories to be cached, separated by newlines.
    cache_on_failure
        Cache even if the build fails.
    cache_key
        Additional cache key to add alongside the automatic job-based key.
    cache_shared_key
        Cache key used instead of the automatic job-based key.
    cache_bin
        Whether to cache ``${CARGO_HOME}/bin``.
    cache_provider
        Cache provider (``github``, ``buildjet``, ``warpbuild``).
    cache_all_crates
        If true, cache all crates; otherwise cache dependent crates.
    cache_workspace_crates
        If true, cache all workspace crates; otherwise dependent crates only.
    matcher
        Enable the Rust problem matcher.
    rustflags
        Set RUSTFLAGS environment variable.
    override
        Setup the last installed toolchain as the default via ``rustup override``.
    rust_src_dir
        Path from root directory to the Rust source directory.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-rust step.

    See Also
    --------
    GitHub repository: https://github.com/actions-rust-lang/setup-rust-toolchain
    """

    recommended_permissions = None

    @classmethod
    def rustc_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['rustc-version']

    @classmethod
    def cargo_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['cargo-version']

    @classmethod
    def rustup_version(cls, id: str) -> StringExpression:
        return context.steps[id].outputs['rustup-version']

    @classmethod
    def cachekey(cls, id: str) -> StringExpression:
        return context.steps[id].outputs.cachekey

    def __new__(
        cls,
        *,
        name: Ostrlike = None,
        version: str = 'v1',
        toolchain: Ostrlike = None,
        target: Ostrlike = None,
        components: list[StringLike] | None = None,
        cache: Oboollike = None,
        cache_directories: list[StringLike] | None = None,
        cache_workspaces: list[StringLike] | None = None,
        cache_on_failure: Oboollike = None,
        cache_key: Ostrlike = None,
        cache_shared_key: Ostrlike = None,
        cache_bin: Oboollike = None,
        cache_provider: Ostrlike = None,
        cache_all_crates: Oboollike = None,
        cache_workspace_crates: Oboollike = None,
        matcher: Oboollike = None,
        rustflags: Ostrlike = None,
        override: Oboollike = None,
        rust_src_dir: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> SetupRust:
        options: dict[str, object] = {
            'toolchain': toolchain,
            'target': target,
            'components': ','.join(str(s) for s in components)
            if components is not None
            else None,
            'cache': cache,
            'cache-directories': '\n'.join(str(s) for s in cache_directories)
            if cache_directories is not None
            else None,
            'cache-workspaces': '\n'.join(str(s) for s in cache_workspaces)
            if cache_workspaces is not None
            else None,
            'cache-on-failure': cache_on_failure,
            'cache-key': cache_key,
            'cache-shared-key': cache_shared_key,
            'cache-bin': cache_bin,
            'cache-provider': validate_choice(
                'cache_provider', cache_provider, ['github', 'buildjet', 'warpbuild']
            ),
            'cache-all-crates': cache_all_crates,
            'cache-workspace-crates': cache_workspace_crates,
            'matcher': matcher,
            'rustflags': rustflags,
            'override': override,
            'rust-src-dir': rust_src_dir,
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Setup Rust'

        return super().__new__(
            cls,
            name,
            'actions-rust-lang/setup-rust-toolchain',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )


class InstallRustTool(ActionStep):
    """Install development tools.

    Parameters
    ----------
    tool
        Tools to install (whitespace or comma separated list).
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    checksum
        Whether to enable checksums.
    fallback
        Whether to use fallback (``none``, ``cargo-binstall``, ``cargo-install``).
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated install-rust-tool step.

    See Also
    --------
    GitHub repository: https://github.com/taiki-e/install-action
    """

    recommended_permissions = None

    def __new__(
        cls,
        *,
        tool: list[StringLike],
        name: Ostrlike = None,
        version: str = 'v2',
        checksum: Oboollike = None,
        fallback: Ostrlike = None,
        args: Ostrlike = None,
        entrypoint: Ostrlike = None,
        condition: Oboolstr = None,
        id: Ostr = None,  # noqa: A002
        env: Mapping[str, StringLike] | None = None,
        continue_on_error: Oboollike = None,
        timeout_minutes: Ointlike = None,
        skip_recommended_permissions: bool = False,
    ) -> InstallRustTool:
        options: dict[str, object] = {
            'tool': ','.join(str(s) for s in tool),
            'checksum': checksum,
            'fallback': validate_choice(
                'fallback', fallback, ['none', 'cargo-binstall', 'cargo-install']
            ),
        }
        options = {key: value for key, value in options.items() if value is not None}

        if name is None:
            name = 'Install Rust Tool'

        return super().__new__(
            cls,
            name,
            'taiki-e/install-action',
            ref=version,
            with_opts=options or None,
            args=args,
            entrypoint=entrypoint,
            condition=condition,
            id=id,
            env=env,
            continue_on_error=continue_on_error,
            timeout_minutes=timeout_minutes,
            skip_recommended_permissions=skip_recommended_permissions,
            recommended_permissions=cls.recommended_permissions,
        )
